import sys
from math import log, exp

from covest import constants
from .inverse import inverse


def print_wrap(x, label='', cond=True):
    if cond:
        print(label, x)
    return x


def verbose_print(message):
    if not constants.VERBOSE:
        return
    sys.stderr.write(message + "\n")


def safe_int(x):
    return int(x) if x != float('inf') else None


def fix_zero(x, val=1):
    if x == 0:
        return val
    else:
        return x


def safe_log(x):
    if x is None or x <= 0:
        return -constants.INF
    return log(x)


def estimate_p(cc, alpha):
    return (cc * (alpha - 1)) / (alpha * cc - alpha - cc)


def kmer_to_read_coverage(coverage, k, r):
    return coverage * r / (r - k + 1)


def fix_coverage(coverage):
    return inverse(lambda c: (c - c * exp(-c)) / (1 - exp(-c) - c * exp(-c)))(coverage)


def nonefloat(x):
    try:
        return float(x)
    except ValueError:
        return None



#! /usr/bin/env python3
import random
import argparse
from os import path
from Bio import SeqIO


BASES = ['A', 'C', 'G', 'T', ]

DEFAULT_FACTOR = 2


def load_reads(fname):
    _, ext = path.splitext(fname)
    fmt = 'fasta'
    if ext == '.fq' or ext == '.fastq':
        fmt = 'fastq'
    with open(fname, "rU") as f:
        for read in SeqIO.parse(f, fmt):
            yield read.id, read.seq


def main(fname, reads_file, factor):
    prob = 1.0 / factor
    with open(fname, 'w') as f:
        for id, read in load_reads(reads_file):
            if random.random() > prob:
                f.write('>{}\n'.format(id))
                f.write('{}\n'.format(read))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Subsample reads randomly form other reads')
    parser.add_argument('-f', '--factor', type=float,
                        default=DEFAULT_FACTOR, help='Factor')
    parser.add_argument('reads', help='Reads file')
    parser.add_argument('output', help='Output file')

    args = parser.parse_args()

    main(args.output, args.reads, args.factor)



from collections import namedtuple
from functools import lru_cache
from os import path

import yaml
from first import first

from covest import __version__
from .models import models, BasicModel
from .utils import safe_int, verbose_print


class InvalidFormatException(Exception):
    def __init__(self, fname):
        self.fname = fname

    def __str__(self):
        return 'Unable to parse %s. Unsupported format.' % self.fname


def load_histogram(fname):
    hist = dict()
    with open(fname, 'r') as f:
        for line in f:
            try:
                l = line.split()
                i = int(l[0])
                cnt = int(l[1])
                hist[i] = cnt
            except (ValueError, KeyError):
                raise InvalidFormatException(fname)
    return hist


@lru_cache(maxsize=None)
def count_reads_size(fname):
    from Bio import SeqIO
    _, ext = path.splitext(fname)
    fmt = 'fasta'
    if ext == '.fq' or ext == '.fastq':
        fmt = 'fastq'
    try:
        with open(fname, "rU") as f:
            return sum(len(read) for read in SeqIO.parse(f, fmt))
    except FileNotFoundError as e:
        verbose_print(e)


def parse_data(f):
    data = yaml.load(f)
    try:
        model_class_name = data['model']
        model = first(models.values(), key=lambda x: x.__name__ == model_class_name)
    except KeyError:
        model = BasicModel
    guess = [
        data.get('guessed_coverage', None),
        data.get('guessed_error_rate', None),
    ]
    estimated = [data.get(k, None) for k in model.params]
    return namedtuple('ParsedData', ('estimated', 'guess', 'model'))(
        estimated=estimated, guess=guess, model=model
    )


def replace_none(dest, src):
    if dest is None or src is None:
        raise ValueError('Invalid arguments.')
    dest = list(dest)
    if len(dest) != len(src):
        raise ValueError('Length of arguments should be equal.')
    for i in range(len(dest)):
        if dest[i] is None:
            dest[i] = src[i]
    return dest


def print_output(
    hist_orig,
    model,
    success,
    sample_factor,
    estimated=None,
    guess=None,
    orig=None,
    reads_size=None,
    silent=False,
):
    def params_to_dict(names, values):
        nonlocal sample_factor
        if values is None or names is None:
            return dict()
        values = [float(v) if v is not None else v for v in values]
        # apply sample factor to coverage
        if values[0] is not None and sample_factor is not None:
            values[0] *= sample_factor
        data = dict()
        for k, v in zip(names, values):
            if v is not None:
                data[k] = v
        return data

    output_data = {
        'model': model.short_name(),
        'hist_size': max(model.hist),
        'sample_factor': sample_factor,
        'success': success,
        'version': __version__,
    }

    if guess is not None:
        output_data.update(params_to_dict(('guessed_coverage', 'guessed_error_rate'), guess))
        output_data['guessed_loglikelihood'] = model.compute_loglikelihood(*guess)
    if estimated is not None:
        output_data.update(params_to_dict(model.params, estimated))
        output_data['loglikelihood'] = model.compute_loglikelihood(*estimated)
        output_data['genome_size'] = safe_int(round(
            sum(
                i * h for i, h in hist_orig.items()
            ) / model.correct_c(estimated[0] * sample_factor)
        ))
        if reads_size is not None:
            output_data['genome_size_reads'] = safe_int(
                round(reads_size / (estimated[0] * sample_factor))
            )
    if orig is not None and any(orig):
        output_data.update(params_to_dict(('provided_%s' % name for name in model.params), orig))
        try:
            output_data['provided_loglikelihood'] = model.compute_loglikelihood(
                *replace_none(orig, estimated)
            )
        except ValueError:
            pass

    if not silent:
        print(yaml.dump(output_data, indent=4, default_flow_style=False))

    return output_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_covest
----------------------------------

Tests for `covest` module.
"""

import unittest

from covest.histogram import compute_coverage_apx, sample_histogram, trim_hist


class TestComputeCovApx(unittest.TestCase):
    def test_compute_coverage_apx(self):
        histogram = {
            1: 2909,
            2: 10891,
            3: 28824,
            4: 56698,
            5: 92099,
            6: 122998,
            7: 137748,
            8: 137507,
            9: 124723,
            10: 100866,
            11: 72467,
            12: 47639,
            13: 29893,
            14: 17119,
            15: 9026,
            16: 4713,
            17: 2077,
            18: 767,
            19: 288,
            20: 139,
            21: 49,
            22: 27,
            23: 37,
            24: 16,
        }
        c, e = compute_coverage_apx(histogram, 21, 100)
        self.assertAlmostEqual(c, 10, delta=1)
        self.assertAlmostEqual(e, 0, delta=0.01)


class TestSampleHistogram(unittest.TestCase):
    def assertDictEqualApprox(self, a, b, threshold):
        for k in set(a) | set(b):
            i = a.get(k, 0)
            j = b.get(k, 0)
            if abs(i - j) > threshold:
                raise AssertionError('{} and {} differs by more then threshold {}'.format(i, j, threshold))

    def test_sample_histogram(self):
        histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            5: 81833,
            6: 37443,
            7: 14628,
            8: 5075,
            9: 1505,
            10: 386,
            11: 133,
            12: 22,
            13: 5,
            14: 9,
            15: 1,
        }
        true_hist = {1: 2949548, 2: 255275, 3: 108173, 4: 36802, 5: 10079, 6: 2303, 7: 452, 8: 79, 9: 12, 10: 3, 11: 1}
        sampled_hist = sample_histogram(histogram, 2)
        self.assertDictEqualApprox(sampled_hist, true_hist, 2)

    def test_sample_sparse_histogram(self):
        histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            6: 37443,
            7: 14628,
            8: 5075,
            12: 22,
            13: 5,
            15: 1,
        }
        true_hist = {1: 2936731, 2: 229577, 3: 82298, 4: 23544, 5: 7025, 6: 1946, 7: 279, 8: 23, 9: 1}
        sampled_hist = sample_histogram(histogram, 2)
        self.assertDictEqualApprox(sampled_hist, true_hist, 2)


class TestTrimHistogram(unittest.TestCase):
    def test_trim_histogram(self):
        histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            5: 81833,
            6: 37443,
            7: 14628,
            8: 5075,
            9: 1505,
            10: 386,
            11: 133,
            12: 22,
            13: 5,
            14: 9,
            15: 1,
        }
        trimmed_hist, tail =trim_hist(histogram, 10)
        self.assertEqual(max(trimmed_hist), 9)
        self.assertEqual(sum(trimmed_hist.values()) + tail, sum(histogram.values()))

    def test_trim_sparse_histogram(self):
        histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            6: 37443,
            7: 14628,
            8: 5075,
            12: 22,
            13: 5,
            15: 1,
        }
        trimmed_hist, tail = trim_hist(histogram, 10)
        self.assertEqual(max(trimmed_hist), 8)
        self.assertEqual(sum(trimmed_hist.values()) + tail, sum(histogram.values()))


if __name__ == '__main__':
    import sys

    sys.exit(unittest.main())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_data_manipulation
----------------------------------

Tests for `covest.data` module.
"""

import unittest
from pathlib import Path

from covest.data import load_histogram, InvalidFormatException


class TestLoadHistogram(unittest.TestCase):
    def setUp(self):
        self.folder = Path(__file__).parent / 'data'

    def test_load_histogram(self):
        true_histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            5: 81833,
            6: 37443,
            7: 14628,
            8: 5075,
            9: 1505,
            10: 386,
            11: 133,
            12: 22,
            13: 5,
            14: 9,
            15: 1,
        }

        hist = load_histogram(str(self.folder / 'simulated_c10_e0.05_r100_k21.hist'))
        self.assertDictEqual(hist, true_histogram)

    def test_load_sparse_histogram(self):
        true_histogram = {
            1: 5311035,
            2: 310620,
            3: 223796,
            4: 150045,
            6: 37443,
            7: 14628,
            8: 5075,
            12: 22,
            13: 5,
            15: 1,
        }

        hist = load_histogram(str(self.folder / 'simulated_c10_e0.05_r100_k21_sparse.hist'))
        self.assertDictEqual(hist, true_histogram)

    def test_load_nonexisting_file(self):
        with self.assertRaises(FileNotFoundError):
            load_histogram('nonexisting_histogram.hist')

    def test_load_invalid_histogram(self):
        with self.assertRaises(InvalidFormatException):
            hist = load_histogram(str(self.folder / 'invalid.hist'))

    def test_load_empty_histogram(self):
        hist = load_histogram(str(self.folder / 'empty.hist'))
        self.assertEqual(len(hist), 0)


if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())



#!/usr/bin/env python
import argparse
import random
from collections import defaultdict
from os import path

from Bio import SeqIO

NS_IGNORE = 0
NS_SINGLE = 1
NS_RANDOM = 2


def single_hash(b):
    return {'a': 0, 'c': 1, 'g': 2, 't': 3}[b]


def hash_kmer(kmer):
    h = 0
    for b in kmer:
        h <<= 2
        h |= single_hash(b)
    return h


def rehash(old_hash, b, k):
    l = 2 * k
    h = old_hash & ((1 << (l - 2)) - 1)
    h <<= 2
    h |= single_hash(b)
    return h


def compute_counts(seq, prev_counts=None, k=20):
    counts = defaultdict(int) if prev_counts is None else prev_counts
    h = hash_kmer(seq[:k])
    counts[h] += 1
    for b in seq[k:]:
        h = rehash(h, b, k)
        counts[h] += 1
    return counts


def preprocess(seq, nstrategy=NS_IGNORE):
    seq = seq.lower()
    if nstrategy == NS_IGNORE:
        seq = ''.join(filter(lambda x: x != 'n', seq))
    elif nstrategy == NS_SINGLE:
        seq = ''.join(('a' if b == 'n' else b) for b in seq)
    elif nstrategy == NS_RANDOM:
        seq = ''.join(random.choice(['a', 'c', 'g', 't']) if b == 'n' else b for b in seq)
    else:
        raise 'Invalid N strategy'
    return seq


def compute_histogram(counts):
    histogram = defaultdict(int)
    max_c = 0
    for c in counts.values():
        histogram[c] += 1
        max_c = max(max_c, c)

    return [histogram[i] for i in range(max_c + 1)]


def load_reads(fname):
    _, ext = path.splitext(fname)
    fmt = 'fasta'
    if ext == '.fq' or ext == '.fastq':
        fmt = 'fastq'
    with open(fname, "rU") as f:
        for read in SeqIO.parse(f, fmt):
            yield read.seq


def main(fname, out_fname, k, n_strategy):
    counts = defaultdict(int)
    for seq in load_reads(fname):
        seq = preprocess(seq, n_strategy)
        counts = compute_counts(seq, prev_counts=counts)
    hist = compute_histogram(counts)

    if out_fname:
        with open(out_fname, 'w') as f:
            for i, v in enumerate(hist):
                f.write('{} {}\n'.format(i, v))
    else:
        print(hist)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Compute kmer_histogram form reads')
    parser.add_argument('-n', '--n-strategy',
                        default='IGNORE', help='Strategy to handle Ns')
    parser.add_argument('-k', type=int,
                        default=20, help='Kmer size')
    parser.add_argument('input', help='input file')
    parser.add_argument('-o', '--output', default=None, help='Output file')

    args = parser.parse_args()

    try:
        n_strategy_d = {
            'IGNORE': NS_IGNORE, 'SINGLE': NS_SINGLE, 'RANDOM': NS_RANDOM
        }
        n_strategy = n_strategy_d[args.n_strategy] if args.n_strategy in n_strategy_d\
            else int(args.n_strategy)
    except:
        n_strategy = NS_IGNORE

    main(args.input, args.output, args.k, n_strategy)



import random
from collections import defaultdict
from covest_poisson import poisson_dist
from math import exp, floor, ceil

from scipy.stats import binom

from covest import constants
from .utils import estimate_p, kmer_to_read_coverage, fix_coverage, verbose_print


def compute_coverage_apx(hist, k, r):
    observed_ones = hist.get(1, 0)
    all_kmers = sum(i * h for i, h in hist.items())
    total_unique_kmers = sum(h for h in hist.values())

    if total_unique_kmers == 0:
        return 0.0, 1.0

    # discard first column
    all_kmers -= observed_ones
    unique_kmers = total_unique_kmers - observed_ones
    # compute coverage from hist >=2
    try:
        cov = all_kmers / unique_kmers
        cov = fix_coverage(cov)
        # fix unique kmers
        unique_kmers /= (1.0 - exp(-cov) - cov * exp(-cov))
        # compute alpha (error read ratio)
        estimated_ones = unique_kmers * cov * exp(-cov)
        estimated_zeros = unique_kmers * exp(-cov)
        error_ones = max(0.0, observed_ones - estimated_ones)
        alpha = error_ones / (total_unique_kmers + estimated_zeros)
        # estimate probability of correct kmer and error rate
        estimated_p = max(0.0, estimate_p(cov, alpha))
        e = 1 - estimated_p ** (1.0 / k)
        # return corrected coverage and error estimate
        if estimated_p > 0:
            # function for conversion between kmer and base coverage
            return float(kmer_to_read_coverage(cov / estimated_p, k, r)), float(e)
        else:
            return 0.0, float(e)
    except ZeroDivisionError:
        return 0.0, 1.0


def sample_histogram(hist, factor=2, trim=None):
    if trim is None:
        if len(hist) > 300:
            trim = get_trim(hist)
        else:
            trim = max(hist)
    else:
        trim = min(max(hist), trim * factor)

    hist = {k: v for k, v in hist.items() if k < trim}
    h = defaultdict(int)
    prob = 1.0 / factor

    for i, v in hist.items():
        if i < 100:
            b = binom(i, prob)
            probs = [b.pmf(j) for j in range(1, i + 1)]
        else:
            probs = poisson_dist(i * prob, i)

        for j, p in enumerate(probs):
            h[j + 1] += v * p

    h = dict(h)
    for i, v in h.items():
        d = v - round(v)
        h[i] = ceil(v) if random.random() < d else floor(v)
    return {k: v for k, v in h.items() if v > 0}   # remove 0 elements


def auto_sample_hist(hist, k, r, trim=None):
    h = dict(hist)
    f = 1
    s = 1
    c, e = compute_coverage_apx(hist, k, r)

    while c > constants.AUTO_SAMPLE_TARGET_COVERAGE:
        f += s
        s *= 2
        h = sample_histogram(hist, factor=f, trim=trim)
        c, e = compute_coverage_apx(h, k, r)

    s //= 4
    f2 = f - s
    while s >= 1:
        h2 = sample_histogram(hist, factor=f2, trim=trim)
        c, e = compute_coverage_apx(h2, k, r)
        if c > constants.AUTO_SAMPLE_TARGET_COVERAGE:
            f2 += s
        else:
            h = h2
            f = f2
            f2 -= s
        s //= 2

    return h, f, c, e


def remove_noise(hist):
    total = sum(hist.values())
    hist_denoised = {k: v for k, v in hist.items() if v / total > constants.NOISE_THRESHOLD}
    return hist_denoised


def get_trim(hist):
    hist = remove_noise(hist)
    ss = float(sum(hist.values()))
    s = 0.0
    trim = max(hist)
    for i, h in sorted(hist.items()):
        s += h
        r = s / ss
        r = round(r, constants.AUTO_TRIM_PRECISION)
        if r == 1:
            trim = i
            break
    return trim


def trim_hist(hist, threshold):
    if threshold >= max(hist):
        return hist, 0
    h = {k: v for k, v in hist.items() if k < threshold}
    tail = sum(v for k, v in hist.items() if k >= threshold)
    # remove 0 elements
    return {k: v for k, v in h.items() if v > 0}, tail


def process_histogram(hist, k, r, trim=None, sample_factor=None):
    hist = dict(hist)
    tail = 0
    if sample_factor is not None and sample_factor > 1:
        verbose_print('Sampling histogram...')
        hist = sample_histogram(hist, sample_factor, trim)
    if sample_factor is None:
        verbose_print('Sampling histogram...')
        hist, sample_factor, c, e = auto_sample_hist(hist, k, r, trim=trim)
        verbose_print('Histogram sampled with factor {}.'.format(sample_factor))
    else:
        c, e = compute_coverage_apx(hist, k, r)
    if trim is None:
        trim = get_trim(hist)
        verbose_print('Trimming at: {}'.format(trim))
        hist, tail = trim_hist(hist, trim)
    elif trim > 0:
        verbose_print('Trimming at: {}'.format(trim))
        hist, tail = trim_hist(hist, trim)
    return hist, tail, sample_factor, c, e



import argparse
from multiprocessing import Pool

from scipy.optimize import minimize

from covest import version_string
from . import constants
from .data import count_reads_size, parse_data, print_output, load_histogram
from .grid import initial_grid, optimize_grid
from .histogram import process_histogram
from .models import select_model, models
from .perf import running_time, running_time_decorator
from .utils import verbose_print, nonefloat


class CoverageEstimator:
    def __init__(self, model, err_scale=1, fix=None):
        self.model = model
        self.fix = fix
        self.err_scale = err_scale
        self.bounds = list(self.model.bounds)
        self.bounds[1] = self.bounds[1][0], self.bounds[1][1] * self.err_scale

    def likelihood_f(self, x):
        args = list(x)
        if self.fix is not None:
            args = [j if self.fix[i] is None else self.fix[i] for i, j in enumerate(args)]
        args[1] /= self.err_scale
        return -self.model.compute_loglikelihood(*args)

    def _optimize(self, r):
        return minimize(
            self.likelihood_f, r,
            method=constants.OPTIMIZATION_METHOD,
            bounds=self.bounds,
            options={'disp': False}
        )

    def compute_coverage(
        self,
        guess,
        starting_points=1,
        use_grid_search=False,
        n_threads=constants.DEFAULT_THREAD_COUNT,
    ):
        r = list(guess)
        r[1] *= self.err_scale
        success = True
        try:
            verbose_print('Bounds: {}'.format(self.bounds))
            if starting_points == 1:
                with running_time('First optimization'):
                    res = self._optimize(r)
                    success = res.success
                    if not success:
                        verbose_print(
                            'Optimization unsuccessful.\n'
                            'Initial params:{}\nResult{}'.format(r, res)
                        )
                    r = res.x
            elif starting_points > 1:
                params = initial_grid(r, count=starting_points, bounds=self.bounds)
                with running_time('Initial grid optimization'):
                    min_r = None
                    with Pool(n_threads) as pool:
                        results = list(pool.map(self._optimize, params))
                    for res in results:
                        if min_r is None or min_r > res.fun:
                            min_r = res.fun
                            success = res.success
                            if not success:
                                verbose_print(
                                    'Optimization unsuccessful.\n'
                                    'Initial params:{}\nResult{}'.format(r, res)
                                )
                            r = res.x

            # If use_grid_search is none, run grid search only on failure
            if use_grid_search is None and not success:
                use_grid_search = True

            if use_grid_search:
                verbose_print('Starting grid search with guess: {}'.format(r))
                r = optimize_grid(
                    self.likelihood_f, r, bounds=self.bounds,
                    fix=self.fix, n_threads=n_threads,
                )
        except KeyboardInterrupt:
            pass
        verbose_print('Estimation finished with status: %s.' % (
            'success' if success else 'failure',
        ))
        r[1] /= self.err_scale
        return r, success


@running_time_decorator
def main(args):
    # Load histogram
    verbose_print('Loading histogram {} with parameters k={} r={}.'.format(
        args.input_histogram, args.kmer_size, args.read_length,
    ))
    hist_orig = load_histogram(args.input_histogram)
    # Process histogram and obtain first guess for c and e
    hist, tail, sample_factor, guess_c, guess_e = process_histogram(
        hist_orig, args.kmer_size, args.read_length,
        trim=args.trim, sample_factor=args.sample_factor,
    )
    reads_size = None
    if args.read_file is not None:
        reads_size = count_reads_size(args.read_file)
    err_scale = args.error_scale
    if sample_factor is None:
        sample_factor = 1
    if args.coverage:
        args.coverage /= sample_factor

    # Model initialisation
    model = select_model(args.model)(
        args.kmer_size, args.read_length, hist, tail,
        max_error=constants.MAX_ERRORS, max_cov=args.max_coverage,
        min_single_copy_ratio=args.min_q1,
    )

    orig = [None] * model.param_count
    for i, v in zip(
        range(model.param_count), (args.coverage, args.error_rate) + tuple(args.params)
    ):
        orig[i] = v
    fix = orig if args.fix else None

    if args.ll_only:
        ll = model.compute_loglikelihood(*orig)
        print('Loglikelihood:', ll)
    else:
        if args.load:  # Load saved data, don't estimate anything
            with open(args.load) as f:
                parsed_data = parse_data(f)
                guess = parsed_data.guess
                res = parsed_data.estimated
        else:
            verbose_print('Estimating coverage...')
            # Compute initial guess
            if args.start_original:
                guess = list(orig)
            else:
                guess = list(model.defaults)
                if not (guess_c == 0 and guess_e == 1):  # We were able to guess cov and e
                    guess[:2] = guess_c, guess_e
                if fix:
                    for i, v in fix:
                        if v is not None:
                            guess[i] = v
            verbose_print('Initial guess: {} ll:{}'.format(
                guess, model.compute_loglikelihood(*guess)
            ))

            # Estimate coverage
            estimator = CoverageEstimator(model, err_scale=err_scale, fix=fix)
            res, success = estimator.compute_coverage(
                guess,
                starting_points=args.starting_points,
                use_grid_search=args.grid,
                n_threads=args.thread_count,
            )

            print_output(
                hist_orig, model, success, sample_factor,
                res, guess, orig,
                reads_size=reads_size,
            )

        # Draw plot
        if args.plot is not None:
            model.plot_probs(
                res, guess, orig, cumulative=args.plot, log_scale=constants.PLOT_LOG_SCALE
            )


def run():
    parser = argparse.ArgumentParser(description='Simulate reads form random genome with errors')
    parser.add_argument('input_histogram', type=str, help='Input histogram')
    parser.add_argument('-v', '--version', action='version', version=version_string,
                        help='Print version and exit.')
    parser.add_argument('-m', '--model', type=str, default='basic',
                        help='Select models for estimation. Options: {}'.format(
                            list(models.keys()))
                        )
    parser.add_argument('-k', '--kmer-size', type=int,
                        default=constants.DEFAULT_K, help='Kmer size')
    parser.add_argument('-r', '--read-length', type=int,
                        default=constants.DEFAULT_READ_LENGTH, help='Read length')
    parser.add_argument('-s', '--genome-size', dest='read_file',
                        help='Calculate genome size from reads')
    parser.add_argument('-sp', '--starting-points', type=int, default=1,
                        help='Number of point to start optimization from.')
    parser.add_argument('-T', '--thread-count', default=constants.DEFAULT_THREAD_COUNT, type=int,
                        help='Thread count')
    parser.add_argument('--plot', type=bool, nargs='?', const=False,
                        help='Plot probabilities (use --plot 1 to plot probs * j)')
    parser.add_argument('--load', type=str, help='Load json')
    parser.add_argument('-t', '--trim', type=int, default=None,
                        help='Trim histogram at this value. '
                             'Set to 0 to disable automatic trimming.')
    parser.add_argument('-sf', '--sample-factor', type=int, default=None,
                        help='Use fixed sample factor for histogram sampling instead of automatic.'
                             ' Set to 1 to not sample at all.')
    parser.add_argument('-g', '--grid', action='store_true', default=False,
                        help='Use grid search for fine-tuning.')
    parser.add_argument('-f', '--fix', action='store_true',
                        help='Fix some params, optimize others')
    parser.add_argument('-c', '--coverage', type=float, help='Coverage')
    parser.add_argument('-M', '--max-coverage', type=int, help='Upper coverage limit')
    parser.add_argument('-e', '--error-rate', type=float, help='Error rate')
    parser.add_argument('-es', '--error-scale', type=float, default=constants.DEFAULT_ERR_SCALE,
                        help='Error scale')
    parser.add_argument('-mq1', '--min-q1', type=float,
                        default=constants.DEFAULT_MIN_SINGLECOPY_RATIO,
                        help='minimum single copy ratio')
    parser.add_argument('-p', '--params', type=nonefloat, nargs='*', default=tuple(),
                        help='Additional model parameters.')
    parser.add_argument('-ll', '--ll-only', action='store_true',
                        help='Only compute log likelihood from provided values')
    parser.add_argument('-so', '--start-original', action='store_true',
                        help='Start optimization form provided values')

    main(parser.parse_args())


if __name__ == '__main__':
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from covest.models import select_model, BasicModel, RepeatsModel


class TestCovest(unittest.TestCase):
    def test_model_selection_fulltext(self):
        self.assertEqual(select_model('basic'), BasicModel)
        self.assertEqual(select_model('repeat'), RepeatsModel)

    def test_model_selection_part(self):
        self.assertEqual(select_model('b'), BasicModel)
        self.assertEqual(select_model('r'), RepeatsModel)

    def test_model_selection_invalid(self):
        with self.assertRaises(ValueError):
            select_model('someInvalidModel')

if __name__ == '__main__':
    import sys
    sys.exit(unittest.main())
